@@ -2,7 +2,7 @@
  * Mp3Splt -- Utility for mp3/ogg splitting without decoding
  *
  * Copyright (c) 2002-2005 M. Trotta - <mtrotta@users.sourceforge.net>
- * Copyright (c) 2005-2008 Alexandru Munteanu - <io_fx@yahoo.fr>
+ * Copyright (c) 2005-2009 Alexandru Munteanu - <io_fx@yahoo.fr>
  *
  * http://mp3splt.sourceforge.net
  *
@@ -29,6 +29,11 @@
 #include <sys/stat.h>
 #include <ctype.h>
 
+#ifdef __WIN32__
+#include <windows.h>
+#include <shlwapi.h>
+#endif
+
 #include <libmp3splt/mp3splt.h>
 
 #include "getopt.h"
@@ -38,10 +43,10 @@
 #ifdef HAVE_CONFIG_H
 #include "config.h"
 #else
-#define VERSION "2.2.1"
+#define VERSION "2.2.3"
 #define PACKAGE_NAME "mp3splt"
 #endif
-#define MP3SPLT_DATE "02/10/08"
+#define MP3SPLT_DATE "18/01/09"
 #define MP3SPLT_AUTHOR1 "Matteo Trotta"
 #define MP3SPLT_AUTHOR2 "Alexandru Munteanu"
 #define MP3SPLT_EMAIL1 "<mtrotta AT users.sourceforge.net>"
@@ -56,6 +61,8 @@ FILE *console_progress = NULL;
 
 //command line options
 typedef struct {
+  //force id3v1 tags, force id3v2 tags
+  short id3v1_option; short id3v2_option;
   //wrap split, list wrap options, error split
   short w_option; short l_option; short e_option;
   //frame mode, cddb/cue option, time split
@@ -66,6 +73,7 @@ typedef struct {
   short o_option; short d_option; short k_option;
   //custom tags, no tags, quiet option
   short g_option; short n_option; short q_option;
+  short N_option;
   //-Q option
   short qq_option;
   //info -i option, m3u file option
@@ -90,6 +98,10 @@ typedef struct {
   int freedb_get_type;
   char freedb_get_server[256];
   int freedb_get_port;
+  //the search string passed in parameter -c query{my artist}
+  char freedb_arg_search_string[2048];
+  //the chosen result passed in parameter : -c query{my artist}[
+  int freedb_arg_result_option;
 } options;
 
 typedef struct
@@ -114,6 +126,10 @@ typedef struct
   //the splitpoints parsed from the arguments
   long *splitpoints;
   int number_of_splitpoints;
+  //command line arguments : on windows, we need to
+  //keep the ones transformed to utf8 and free them later
+  char **argv;
+  int argc;
 } main_data;
 
 //we make a global variable, we use it in
@@ -127,6 +143,36 @@ void free_options(options **opt)
   {
     if (*opt)
     {
+      if ((*opt)->dir_arg)
+      {
+        free((*opt)->dir_arg);
+        (*opt)->dir_arg = NULL;
+      }
+
+      if ((*opt)->cddb_arg)
+      {
+        free((*opt)->cddb_arg);
+        (*opt)->cddb_arg = NULL;
+      }
+
+      if ((*opt)->m3u_arg)
+      {
+        free((*opt)->m3u_arg);
+        (*opt)->m3u_arg = NULL;
+      }
+
+      if ((*opt)->param_args)
+      {
+        free((*opt)->param_args);
+        (*opt)->param_args = NULL;
+      }
+
+      if ((*opt)->custom_tags)
+      {
+        free((*opt)->custom_tags);
+        (*opt)->custom_tags = NULL;
+      }
+
       if ((*opt)->output_format)
       {
         free((*opt)->output_format);
@@ -175,6 +221,25 @@ void free_main_struct(main_data **d)
         data->splitpoints = NULL;
       }
 
+#ifdef __WIN32__
+      //free argv
+      if (data->argv)
+      {
+        int i = 0;
+        for (i = 0; i < data->argc;i++)
+        {
+          if (data->argv[i])
+          {
+            free(data->argv[i]);
+            data->argv[i] = NULL;
+          }
+        }
+
+        free(data->argv);
+        data->argv = NULL;
+      }
+#endif 
+
       //free left variables in the state
       mp3splt_free_state(data->state, NULL);
       data->state = NULL;
@@ -248,14 +313,15 @@ void show_small_help_exit(main_data *data)
 {
   free_main_struct(&data);
   print_message("\n"
-      "USAGE (Please read man page for complete documentation)\n"
+      "USAGE: (Please read man page for complete documentation)\n"
       "      mp3splt [OPTIONS] FILE1 [FILE2] ... [BEGIN_TIME] [TIME] ... [END_TIME]\n"
-      "      TIME FORMAT: min.sec[.0-99], even if minutes are over 59 (or EOF for End Of File). \n"
+      "      TIME FORMAT: min.sec[.0-99], even if minutes are over 59\n"
+      "                   (or EOF for End Of File). \n"
       "\nOPTIONS (split mode options)\n"
-      //"\tIf you have a ogg stream, split from 0 to a big number to fix it; \n\t example : mp3splt stream_song.ogg 0.0 7000.0\n"
       " -t + TIME: to split files every fixed time len. (TIME format same as above). \n"
-      " -c + file.cddb, file.cue or \"query\". Get splitpoints and filenames from a\n"
-      "      .cddb or .cue file or from Internet (\"query\"). Use -a to auto-adjust.\n"
+      " -c + file.cddb, file.cue or \"query\" or \"query{album}\". Get splitpoints and\n"
+      "      filenames from a .cddb or .cue file or from Internet (\"query\").\n"
+      "      Use -a to auto-adjust splitpoints.\n"
       " -s   Silence detection: automatically find splitpoint. (Use -p for arguments)\n"
       " -w   Splits wrapped files created with Mp3Wrap or AlbumWrap.\n"
       " -l   Lists the tracks from file without extraction. (Only for wrapped mp3)\n"
@@ -265,23 +331,30 @@ void show_small_help_exit(main_data *data)
       " -v   Prints current version and exits\n"
       " -h   Shows this help\n"
       "\n(other options)\n"
+      " -1   For mp3 files, force output tags as version 1\n"
+      " -2   For mp3 files, force output tags as version 2\n"
+      "      (default is to set the same version as the file to split)\n"
       " -m + M3U_FILE: Appends to the specified m3u file the split filenames.\n"
       " -f   Frame mode (mp3 only): process all frames. For higher precision and VBR.\n"
       " -a   Auto-Adjust splitpoints with silence detection. (Use -p for arguments)\n"
       " -p + PARAMETERS (th, nt, off, min, rm, gap): user arguments for -s and -a.\n"
       " -o + FORMAT: output filename pattern. Can contain those variables:\n"
       "      @a: artist tag, @p: performer tag (might not exists), @b: album tag\n"
-      "      @t: title tag, @n: track number tag, @f: original filename\n"
+      "      @t: title tag, @n: track number tag (a digit may follow the 'n' for\n"
+      "      the number of digits to output), @f: original filename\n"
       " -g + TAGS : custom tags for the split files.\n"
       "      TAGS can contain those variables : \n"
-      "         @a, @b, @t, @y, @c, @n and @o (set original tags).\n"
-      "      TAGS format is like [@a=artist1,@t=title1]\%[@o,@a=artist2]\n"
+      "         @a, @b, @t, @y, @c, @n, @o (set original tags),\n"
+      "         @N (auto increment track number).\n"
+      "      TAGS format is like [@a=artist1,@t=title1]\%[@o,@N=2,@a=artist2]\n"
       "       (\% means that we set the tags for all remaining files)\n"
       " -d + DIRNAME: to put all output files in the directory DIRNAME.\n"
       " -k   Consider input not seekable (slower). Default when input is STDIN (-).\n"
       " -n   No Tag: does not write ID3v1 or vorbis comment. If you need clean files.\n"
+      " -N   Don't create the 'mp3splt.log' log file when using '-s'.\n"
       " -q   Quiet mode: try not prompt (if possible) and print less messages.\n"
-      " -Q   Very quiet mode: don't print anything to stdout and no progress bar (also enables -q).\n"
+      " -Q   Very quiet mode: don't print anything to stdout and no progress bar\n"
+      "       (also enables -q).\n"
       " -D   Debug mode: used to debug the program (by developers).\n\n"
       "      Read man page for complete documentation.\n");
 
@@ -296,8 +369,20 @@ void show_small_help_exit(main_data *data)
 }
 
 //
-char **rmopt (char **argv, int offset, int tot)
+char **rmopt(char **argv, int offset, int tot)
 {
+#ifdef __WIN32__
+  int i = 0;
+  for (i = 0; i < offset;i++)
+  {
+    if (argv[i])
+    {
+      free(argv[i]);
+      argv[i] = NULL;
+    }
+  }
+#endif
+
   char **first = &argv[1];
   while (offset < tot)
   {
@@ -309,24 +394,6 @@ char **rmopt (char **argv, int offset, int tot)
   return argv;
 }
 
-//removes the element index from the argv
-char **rmopt2 (char **argv,int index, int tot)
-{
-  char **first = &argv[1];
-  int i = 1;
-  while (i < tot)
-  {
-    if (i != index)
-    {
-      *first = argv[i];
-      first++;
-    }
-    i++;
-  }
-
-  return argv;
-}
-
 //parse the -p option
 int parse_arg(char *arg, float *th, int *gap,
     int *nt, float *off, int *rm, float *min)
@@ -575,9 +642,9 @@ void check_args(int argc, main_data *data)
     //no tags (-n)
     if (opt->n_option)
     {
-      if (opt->i_option)
+      if (opt->i_option || opt->id3v1_option || opt->id3v2_option)
       {
-        print_error_exit("the -n option cannot be used with -i", data);
+        print_error_exit("the -n option cannot be used with -i or -1 or -2", data);
       }
     }
 
@@ -620,6 +687,15 @@ void check_args(int argc, main_data *data)
         }
       }
     }
+
+    if (opt->N_option)
+    {
+      if (!opt->s_option)
+      {
+        print_error_exit("the -N option must be used with"
+            " silence detection (-s option)", data);
+      }
+    }
   }
 }
 
@@ -647,7 +723,7 @@ void process_confirmation_error(int conf, main_data *data)
   }
   if (conf == SPLT_DEWRAP_OK)
   {
-    print_message("\nAll files have been splitted correctly. Visit http://mp3wrap.sourceforge.net!");
+    print_message("\nAll files have been split correctly. Visit http://mp3wrap.sourceforge.net!");
   }
 }
 
@@ -697,42 +773,26 @@ long c_hundreths(const char *s)
   return hun;
 }
 
-//interaction with the user when he was choosing the
-//freedb.org search to detect if the entered string was a number
-int checkstring(const char *s)
-{
-  int i;
-  for (i=0; i<strlen(s); i++)
-  {
-    if ((isalnum(s[i]) == 0) && (s[i] != 0x20))
-    {
-      fprintf(console_err," Error: '%c' is not allowed !", s[i]);
-      fflush(console_err);
-      return -1;
-    }
-  }
-
-  return 0;
-}
-
 //for the moment 2 ways of getting the file and one way to search it
 //freedb get type can be : cddb_cgi or cddb_protocol
 //freedb search type can be : cddb_cgi or web_search
 //query[get=cddb_cgi://freedb2.org/~cddb/cddb.cgi:80,search=cddb_cgi://freedb2.org/~cddb/cddb.cgi:80]
 //query[get=cddb_protocol://freedb.org:8880,search=cddb_cgi://freedb2.org/~cddb/cddb.cgi:80]
 //query[get=cddb_cgi://freedb.org/~cddb/cddb.cgi:80,search=cddb_cgi://freedb2.org/~cddb/cddb.cgi:80]
+//query[get...]{search_string}
+//query[get...]{search_string}(chosen_result_int)
 //we parse the query arguments
 int parse_query_arg(options *opt, const char *query)
 {
   const char *cur_pos = NULL;
   const char *end_pos = NULL;
   const char *test_pos = NULL;
-  cur_pos = query;
+  cur_pos = query + 5;
 
   short ambigous = SPLT_FALSE;
 
-  //if we have query[
-  if (strstr(query ,"query[") == query)
+  //if we have [get=...]
+  if (cur_pos[0] == '[')
   {
     cur_pos = strchr(query,'[');
 
@@ -741,14 +801,6 @@ int parse_query_arg(options *opt, const char *query)
     {
       ambigous = SPLT_TRUE;
     }
-    else
-    {
-      //if we have something after ], ambigous
-      if (*(test_pos+1) != '\0')
-      {
-        ambigous = SPLT_TRUE;
-      }
-    }
 
     int search_found = SPLT_FALSE;
     int get_found = SPLT_FALSE;
@@ -808,7 +860,7 @@ int parse_query_arg(options *opt, const char *query)
         if (end_pos-cur_pos < 255)
         {
           snprintf(freedb_type,end_pos-cur_pos+1,"%s",cur_pos);
-          freedb_type[end_pos-cur_pos+1] = '\0';
+          freedb_type[end_pos-cur_pos] = '\0';
           if ((strchr(cur_pos,',')==end_pos) ||
               (strchr(cur_pos,']')==end_pos))
           {
@@ -837,7 +889,7 @@ int parse_query_arg(options *opt, const char *query)
           && (*end_pos != '\0'))
       {
         snprintf(freedb_server,end_pos-cur_pos+1,"%s",cur_pos);
-        freedb_server[end_pos-cur_pos+1] = '\0';
+        freedb_server[end_pos-cur_pos] = '\0';
         if (*(end_pos+1) == ']' || *(end_pos+1) == ',')
         {
           cur_pos = end_pos+1;
@@ -877,7 +929,7 @@ int parse_query_arg(options *opt, const char *query)
         if ((end_pos != cur_pos) && (end_pos-cur_pos < 10))
         {
           snprintf(freedb_port,end_pos-cur_pos+1,"%s",cur_pos);
-          freedb_port[end_pos-cur_pos+1] = '\0';
+          freedb_port[end_pos-cur_pos] = '\0';
           cur_pos = end_pos;
         }
 
@@ -1005,6 +1057,61 @@ int parse_query_arg(options *opt, const char *query)
     }
   }
 
+  //possible search string
+  int we_have_search_string = SPLT_FALSE;
+  if (cur_pos != NULL)
+  {
+    if ((cur_pos = strchr(cur_pos, '{')))
+    {
+      //if we don't have }, ambigous
+      if (!(end_pos = strchr(cur_pos,'}')))
+      {
+        ambigous = SPLT_TRUE;
+      }
+      else
+      {
+        if (end_pos-cur_pos < 2048)
+        {
+          snprintf(opt->freedb_arg_search_string, end_pos-cur_pos, "%s", cur_pos+1);
+          opt->freedb_arg_search_string[end_pos-cur_pos-1] = '\0';
+          we_have_search_string = SPLT_TRUE;
+        }
+      }
+    }
+
+    if (ambigous)
+    {
+      return ambigous;
+    }
+  }
+
+  //possible get result integer from the search results
+  if (cur_pos != NULL)
+  {
+    if ((cur_pos = strchr(cur_pos, '(')))
+    {
+      if (!we_have_search_string)
+      {
+        return ambigous;
+      }
+  
+      //if we don't have ), ambigous
+      if (!(end_pos = strchr(cur_pos,')')))
+      {
+        ambigous = SPLT_TRUE;
+      }
+      else
+      {
+        char chosen_int[256] = { '\0' };
+        if (end_pos-cur_pos < 256)
+        {
+          snprintf(chosen_int, end_pos-cur_pos, "%s", cur_pos+1);
+          opt->freedb_arg_result_option = atoi(chosen_int);
+        }
+      }
+    }
+  }
+
   return ambigous;
 }
 
@@ -1043,64 +1150,103 @@ void do_freedb_search(main_data *data)
       get_type,opt->freedb_get_server,opt->freedb_get_port);
   fflush(console_out);
 
-  //freedb search
-  print_message("CDDB QUERY. Insert album and artist informations to find cd.");
+  char *freedb_search_string = NULL;
+  char freedb_input[2048] = { '\0' };
+  //if we haven't chosen to search from the arguments, interactive search
+  if (opt->freedb_arg_search_string[0] == '\0')
+  {
+    //freedb search
+    print_message("CDDB QUERY. Insert album and artist informations to find cd.");
 
-  char freedb_input[1024];
-  short first_time = SPLT_TRUE;
-  //here we search freedb
-  do {
-    if (!first_time)
-    {
-      print_message("\nPlease search something ...");
-    }
+    short first_time = SPLT_TRUE;
+    //here we search freedb
+    do {
+      if (!first_time)
+      {
+        print_message("\nPlease search something ...");
+      }
 
-    fprintf(console_out, "\n\t____________________________________________________________]");
-    fprintf(console_out, "\r Search: [");
-    fgets(freedb_input, 800, stdin);
+      memset(freedb_input, '\0', sizeof(freedb_input));
 
-    first_time = SPLT_FALSE;
+      fprintf(console_out, "\n\t____________________________________________________________]");
+      fprintf(console_out, "\r Search: [");
 
-    freedb_input[strlen(freedb_input)-1]='\0';
-  } while ((strlen(freedb_input)==0)||
-      (checkstring(freedb_input)!=0));
+      fgets(freedb_input, 2046, stdin);
 
+      first_time = SPLT_FALSE;
+
+      freedb_input[strlen(freedb_input)-1] = '\0';
+
+    } while (strlen(freedb_input)==0);
+
+    freedb_search_string = freedb_input;
+  }
+  else
+  {
+    freedb_search_string = opt->freedb_arg_search_string;
+  }
+
+  fprintf(console_out, "\n  Search string : %s\n",freedb_search_string);
   fprintf(console_out, "\nSearching from %s on port %d using %s ...\n",
       opt->freedb_search_server,opt->freedb_search_port, search_type);
   fflush(console_out);
 
   //the freedb results
-  const splt_freedb_results *f_results;
+  const splt_freedb_results *f_results = NULL;
   //we search the freedb
-  f_results = mp3splt_get_freedb_search(state,freedb_input,
+  f_results = mp3splt_get_freedb_search(state, freedb_search_string,
       &err, opt->freedb_search_type,
       opt->freedb_search_server,
       opt->freedb_search_port);
   process_confirmation_error(err, data);
 
-  //print the searched informations
-  print_message("List of found cd:");
+  //if we don't have an auto-select the result X from the arguments :
+  // (query{artist}(resultX)
+  //, then interactive user ask
+  int selected_cd = 0;
+  if (opt->freedb_arg_result_option < 0)
+  {
+    //print the searched informations
+    print_message("List of found cd:");
 
-  int cd_number = 0;
-  short end = SPLT_FALSE;
-  do {
-    fprintf(console_out,"%3d) %s\n",
-        f_results->results[cd_number].id,
-        f_results->results[cd_number].name);
+    int cd_number = 0;
+    short end = SPLT_FALSE;
+    do {
+      fprintf(console_out,"%3d) %s\n",
+          f_results->results[cd_number].id,
+          f_results->results[cd_number].name);
 
-    int i = 0;
-    for(i = 0; i < f_results->results[cd_number].revision_number; i++)
-    {
-      fprintf(console_out, "  |\\=>");
-      fprintf(console_out, "%3d) ", f_results->results[cd_number].id+i+1);
-      fprintf(console_out, "Revision: %d\n", i+2);
-
-      //break at 22
-      if (((f_results->results[cd_number].id+i+2)%22)==0)
+      int i = 0;
+      for(i = 0; i < f_results->results[cd_number].revision_number; i++)
       {
-        //duplicate, see below
+        fprintf(console_out, "  |\\=>");
+        fprintf(console_out, "%3d) ", f_results->results[cd_number].id+i+1);
+        fprintf(console_out, "Revision: %d\n", i+2);
+
+        //break at 22
+        if (((f_results->results[cd_number].id+i+2)%22)==0)
+        {
+          //duplicate, see below
+          char junk[18];
+          fprintf(console_out, "-- 'q' to select cd, Enter for more:");
+          fflush(console_out);
+
+          fgets(junk, 16, stdin);
+          if (junk[0]=='q')
+          {
+            end = SPLT_TRUE;
+            goto end;
+          }
+        }
+      }
+
+      //we read result from the char, q tu select cd or
+      //enter to show more results
+      if (((f_results->results[cd_number].id+1)%22)==0)
+      {
+        //duplicate, see ^^
         char junk[18];
-        fprintf(console_out, "-- 'q' to select cd, Enter for more:");
+        fprintf(console_out, "-- 'q' to select cd, Enter for more: ");
         fflush(console_out);
 
         fgets(junk, 16, stdin);
@@ -1110,72 +1256,62 @@ void do_freedb_search(main_data *data)
           goto end;
         }
       }
-    }
-
-    //we read result from the char, q tu select cd or
-    //enter to show more results
-    if (((f_results->results[cd_number].id+1)%22)==0)
-    {
-      //duplicate, see ^^
-      char junk[18];
-      fprintf(console_out, "-- 'q' to select cd, Enter for more: ");
-      fflush(console_out);
-
-      fgets(junk, 16, stdin);
-      if (junk[0]=='q')
-      {
-        end = SPLT_TRUE;
-        goto end;
-      }
-    }
 
 end:
-    if (end)
-    {
-      end = SPLT_FALSE;
-      break;
-    }
-
-    cd_number++;
-  } while (cd_number < f_results->number);
-
-  //select the CD
-  //input of the selected cd
-  char sel_cd_input[1024];
-  //selected_cd = the selected cd
-  int selected_cd = 0,tot = 0;
-  do {
-    selected_cd = 0;
-    fprintf(console_out, "Select cd #: ");
-    fflush(console_out);
-    fgets(sel_cd_input, 254, stdin);
-    sel_cd_input[strlen(sel_cd_input)-1]='\0';
-    tot = 0;
-
-    if (sel_cd_input[tot] == '\0') 
-    {
-      selected_cd = -1;
-    }
-
-    while(sel_cd_input[tot] != '\0')
-    {
-      if (isdigit(sel_cd_input[tot++])==0)
+      if (end)
       {
-        fprintf(console_out, "Please ");
-        fflush(console_out);
-
-        selected_cd = -1;
+        end = SPLT_FALSE;
         break;
       }
-    }
 
-    if (selected_cd != -1) 
+      cd_number++;
+    } while (cd_number < f_results->number);
+
+    //select the CD
+    //input of the selected cd
+    char sel_cd_input[1024];
+    int tot = 0;
+    do {
+      selected_cd = 0;
+      fprintf(console_out, "Select cd #: ");
+      fflush(console_out);
+      fgets(sel_cd_input, 254, stdin);
+      sel_cd_input[strlen(sel_cd_input)-1]='\0';
+      tot = 0;
+
+      if (sel_cd_input[tot] == '\0') 
+      {
+        selected_cd = -1;
+      }
+
+      while(sel_cd_input[tot] != '\0')
+      {
+        if (isdigit(sel_cd_input[tot++])==0)
+        {
+          fprintf(console_out, "Please ");
+          fflush(console_out);
+
+          selected_cd = -1;
+          break;
+        }
+      }
+
+      if (selected_cd != -1) 
+      {
+        selected_cd = atoi (sel_cd_input);
+      }
+
+    } while ((selected_cd >= f_results->number) 
+        || (selected_cd < 0));
+  }
+  else
+  {
+    selected_cd = opt->freedb_arg_result_option;
+    if (selected_cd >= f_results->number)
     {
-      selected_cd = atoi (sel_cd_input);
+      selected_cd = 0;
     }
-
-  } while ((selected_cd >= f_results->number) 
-      || (selected_cd < 0));
+  }
 
   fprintf(console_out, "\nGetting file from %s on port %d using %s ...\n",
       opt->freedb_get_server,opt->freedb_get_port, get_type);
@@ -1186,10 +1322,6 @@ end:
       MP3SPLT_CDDBFILE, &err, opt->freedb_get_type,
       opt->freedb_get_server, opt->freedb_get_port);
   process_confirmation_error(err, data);
-
-  //we get the splitpoints from the file
-  mp3splt_put_cddb_splitpoints_from_file(state, MP3SPLT_CDDBFILE, &err);
-  process_confirmation_error(err, data);
 }
 
 //prints a library message
@@ -1293,6 +1425,7 @@ options *new_options(main_data *data)
 {
   options *opt = my_malloc(sizeof(options), data);
 
+  opt->id3v1_option = SPLT_FALSE; opt->id3v2_option = SPLT_FALSE;
   opt->w_option = SPLT_FALSE; opt->l_option = SPLT_FALSE;
   opt->e_option = SPLT_FALSE; opt->f_option = SPLT_FALSE;
   opt->c_option = SPLT_FALSE; opt->t_option = SPLT_FALSE;
@@ -1301,6 +1434,7 @@ options *new_options(main_data *data)
   opt->d_option = SPLT_FALSE; opt->k_option = SPLT_FALSE;
   opt->g_option = SPLT_FALSE; opt->n_option = SPLT_FALSE;
   opt->q_option = SPLT_FALSE; opt->i_option = SPLT_FALSE;
+  opt->N_option = SPLT_FALSE;
   opt->qq_option = SPLT_FALSE;
   opt->m_option = SPLT_FALSE;
   opt->cddb_arg = NULL; opt->dir_arg = NULL;
@@ -1319,6 +1453,9 @@ options *new_options(main_data *data)
   snprintf(opt->freedb_get_server,255, "%s", SPLT_FREEDB2_CGI_SITE);
   opt->freedb_get_port = SPLT_FREEDB_CDDB_CGI_PORT;
 
+  opt->freedb_arg_search_string[0] = '\0';
+  opt->freedb_arg_result_option = -1;
+
   return opt;
 }
 
@@ -1385,7 +1522,7 @@ int check_if_directory(char *fname)
   return SPLT_FALSE;
 }
 
-void get_silence_level(float level, void *user_data)
+void get_silence_level(long time, float level, void *user_data)
 {
   silence_level *sl = user_data;
   if (level == INT_MIN)
@@ -1447,7 +1584,45 @@ void append_splitpoint(main_data *data, long value)
   }
 }
 
-main_data *create_main_struct()
+#ifdef __WIN32__
+char **win32_get_utf8_args(main_data *data)
+{
+  char **argv_utf8 = my_malloc(sizeof(char *) * data->argc, data);
+  LPWSTR *argv_utf16 = NULL;
+  int nArgs = 0;
+  int i = 0;
+
+  argv_utf16 = CommandLineToArgvW(GetCommandLineW(), &nArgs);
+  if (argv_utf16 == NULL )
+  {
+    if (argv_utf8)
+    {
+      free(argv_utf8);
+      argv_utf8 = NULL;
+    }
+
+    print_error_exit("CommandLineToArgvW failed (oh !)", data);
+  }
+  else
+  {
+    for (i=0; i<nArgs; i++)
+    {
+      argv_utf8[i] = mp3splt_win32_utf16_to_utf8((wchar_t *)argv_utf16[i]);
+      if (argv_utf8[i] == NULL)
+      {
+        print_error_exit("failed to allocate argv_utf8 memory", data);
+      }
+    }
+
+    LocalFree(argv_utf16);
+    argv_utf16 = NULL;
+  }
+
+  return argv_utf8;
+}
+#endif
+
+main_data *create_main_struct(int argc, char **orig_argv)
 {
   main_data *data = NULL;
   data = my_malloc(sizeof(main_data), data);
@@ -1467,11 +1642,19 @@ main_data *create_main_struct()
   data->sl->number_of_levels = 0;
   data->sl->print_silence_level = SPLT_TRUE;
 
+  data->argc = argc;
+#ifdef __WIN32__
+  data->argv = NULL;
+  data->argv = win32_get_utf8_args(data);
+#else
+  data->argv = orig_argv;
+#endif
+
   return data;
 }
 
 //main program starts here
-int main(int argc, char *argv[])
+int main(int argc, char **orig_argv)
 {
   console_out = stdout;
   console_err = stderr;
@@ -1480,7 +1663,7 @@ int main(int argc, char *argv[])
   //possible error
   int err = SPLT_OK;
 
-  main_data *data = create_main_struct();
+  main_data *data = create_main_struct(argc, orig_argv);
 
   //we create our state
   data->state = mp3splt_new_state(&err);
@@ -1493,12 +1676,6 @@ int main(int argc, char *argv[])
   //close nicely on Ctrl+C (for example)
   signal(SIGINT, sigint_handler);
 
-  //enable logging the silence splitpoints in a file
-  mp3splt_set_int_option(state, SPLT_OPT_ENABLE_SILENCE_LOG, SPLT_TRUE);
-  //silence splitpoints log filename
-  mp3splt_set_silence_log_filename(state, "mp3splt.log");
-  process_confirmation_error(err, data);
-
   //callback for the library messages
   mp3splt_set_message_function(state, put_library_message);
   mp3splt_set_silence_level_function(state, get_silence_level, data->sl);
@@ -1514,13 +1691,21 @@ int main(int argc, char *argv[])
   //parse command line options
   int option;
   //I have erased the "-i" option
-  while ((option = getopt(argc, argv, "m:SDvifkwleqnasc:d:o:t:p:g:hQ")) != -1)
+  while ((option = getopt(data->argc, data->argv, "m:SDvifkwleqnasc:d:o:t:p:g:hQN12")) != -1)
   {
     switch (option)
     {
       case 'h':
         show_small_help_exit(data);
         break;
+      case '1':
+        mp3splt_set_int_option(state, SPLT_OPT_FORCE_TAGS_VERSION, 1);
+        opt->id3v1_option = SPLT_TRUE;
+        break;
+      case '2':
+        mp3splt_set_int_option(state, SPLT_OPT_FORCE_TAGS_VERSION, 2);
+        opt->id3v2_option = SPLT_TRUE;
+        break;
       case 'D':
         mp3splt_set_int_option(state, SPLT_OPT_DEBUG_MODE, SPLT_TRUE);
         break;
@@ -1572,17 +1757,20 @@ int main(int argc, char *argv[])
         //default tags
         mp3splt_set_int_option(state, SPLT_OPT_TAGS, SPLT_CURRENT_TAGS);
         opt->c_option = SPLT_TRUE;
-        opt->cddb_arg = optarg;
+        opt->cddb_arg = strdup(optarg);
         break;
       case 'm':
         opt->m_option = SPLT_TRUE;
-        opt->m3u_arg = optarg;
+        opt->m3u_arg = strdup(optarg);
         mp3splt_set_m3u_filename(state, opt->m3u_arg);
         break;
       case 'd':
-        opt->dir_arg = optarg;
+        opt->dir_arg = strdup(optarg);
         opt->d_option = SPLT_TRUE;
         break;
+      case 'N':
+        opt->N_option = SPLT_TRUE;
+        break;
       case 'o':
         //default output is false now
         mp3splt_set_int_option(state, SPLT_OPT_OUTPUT_FILENAMES, SPLT_OUTPUT_FORMAT);
@@ -1620,11 +1808,11 @@ int main(int argc, char *argv[])
         break;
       case 'p':
         opt->p_option = SPLT_TRUE;
-        opt->param_args = optarg;
+        opt->param_args = strdup(optarg);
         break;
       case 'g':
         mp3splt_set_int_option(state, SPLT_OPT_TAGS, SPLT_CURRENT_TAGS);
-        opt->custom_tags = optarg;
+        opt->custom_tags = strdup(optarg);
         opt->g_option = SPLT_TRUE;
         break;
       case 'Q':
@@ -1647,65 +1835,6 @@ int main(int argc, char *argv[])
     print_version_authors(console_err);
   }
 
-  //if -o option, then take the directory path and set it as dir_char
-  //if -d option is also specified, then the -d option will replace this
-  //path
-  if (!opt->d_option)
-  {
-    if (opt->o_option)
-    {
-      if (opt->output_format)
-      {
-        char *dup = strdup(opt->output_format);
-        if (!dup)
-        {
-          print_error_exit("cannot allocate memory !",data);
-        }
-
-        int replace_output_format = SPLT_FALSE;
-        int malloc_size = 0;
-        char *p = NULL;
-        //if -o argument is a directory
-        if (check_if_directory(dup))
-        {
-          replace_output_format = SPLT_TRUE;
-          opt->o_option = SPLT_FALSE;
-        }
-        else
-        {
-          //if not a directory, find the first dirchar from the end
-          if ((p = strrchr(dup,SPLT_DIRCHAR)) != NULL) 
-          {
-            malloc_size = strlen(p) + 1;
-            replace_output_format = SPLT_TRUE;
-          }
-        }
-
-        //if we replace the output format
-        if (replace_output_format)
-        {
-          free(opt->output_format);
-          opt->output_format = NULL;
-          //if we really replace the output format
-          if (malloc_size != 0)
-          {
-            opt->output_format = my_malloc(sizeof(char) * malloc_size, data);
-          }
-          //for 'strrchr' version
-          if (malloc_size != 0)
-          {
-            snprintf(opt->output_format,malloc_size,"%s",p);
-            *p = '\0';
-          }
-          err = mp3splt_set_path_of_split(state, dup);
-          process_confirmation_error(err, data);
-        }
-        free(dup);
-        dup = NULL;
-      }
-    }
-  }
-
   //if -n option, set no tags whatever happends
   if (opt->n_option)
   {
@@ -1714,6 +1843,45 @@ int main(int argc, char *argv[])
 
   err = SPLT_OK;
 
+  //add special directory search for plugins on Windows
+#ifdef __WIN32__
+  //add the directory of the executable in the plugin scan directories
+  char *executable = strdup(data->argv[0]);
+  char *end = strrchr(executable, SPLT_DIRCHAR);
+  if (end)
+  {
+    *end = '\0';
+    mp3splt_append_plugins_scan_dir(state, executable);
+  }
+  if (executable)
+  {
+    free(executable);
+    executable = NULL;
+  }
+
+  //also add the installation directory that we take from the registry
+  char mp3splt_uninstall_file[2048] = { '\0' };
+  DWORD dwType, dwSize = sizeof(mp3splt_uninstall_file) - 1;
+  if (SHGetValue(HKEY_LOCAL_MACHINE,
+        TEXT("SOFTWARE\\mp3splt"),
+        TEXT("UninstallString"),
+        &dwType,
+        mp3splt_uninstall_file,
+        &dwSize) != ERROR_SUCCESS)
+  {
+    //do nothing if error
+  }
+  else
+  {
+    end = strrchr(mp3splt_uninstall_file, SPLT_DIRCHAR);
+    if (end)
+    {
+      *end = '\0';
+      mp3splt_append_plugins_scan_dir(state, mp3splt_uninstall_file);
+    }
+  }
+#endif
+
   //after getting the options (especially the debug option), find plugins
   err = mp3splt_find_plugins(state);
   process_confirmation_error(err, data);
@@ -1765,6 +1933,9 @@ int main(int argc, char *argv[])
   int output_format_error = SPLT_OK;
   if (opt->o_option)
   {
+    //enable to create directories from the output filenames
+    mp3splt_set_int_option(state, SPLT_OPT_CREATE_DIRS_FROM_FILENAMES, SPLT_TRUE);
+
     //we set our output format
     mp3splt_set_oformat(state, opt->output_format,&output_format_error);
     if (output_format_error != SPLT_OUTPUT_FORMAT_AMBIGUOUS)
@@ -1775,13 +1946,22 @@ int main(int argc, char *argv[])
 
   if (optind > 1)
   {
-    argv = rmopt(argv, optind, argc);
-    argc -= optind-1;
+    data->argv = rmopt(data->argv, optind, data->argc);
+    data->argc -= optind-1;
   }
 
   //check arguments
   check_args(argc, data);
 
+  //enable/disable logging the silence splitpoints in a file
+  mp3splt_set_int_option(state, SPLT_OPT_ENABLE_SILENCE_LOG, ! opt->N_option);
+  //silence splitpoints log filename
+  if (! opt->N_option)
+  {
+    mp3splt_set_silence_log_filename(state, "mp3splt.log");
+    process_confirmation_error(err, data);
+  }
+
   int i = 0;
   data->filenames = NULL;
   data->number_of_filenames = 0;
@@ -1789,9 +1969,9 @@ int main(int argc, char *argv[])
   data->number_of_splitpoints = 0;
   char *pointer = NULL;
   //we get out the filenames and the splitpoints from the left arguments
-  for (i=1; i < argc; i++)
+  for (i=1; i < data->argc; i++)
   {
-    pointer = argv[i];
+    pointer = data->argv[i];
     long hundreths = c_hundreths(pointer);
     if (hundreths != -1)
     {
@@ -1887,15 +2067,22 @@ int main(int argc, char *argv[])
           else
           {
             //if we have a freedb search
-            if ((strcmp(opt->cddb_arg, "query")==0)
-                ||(strstr(opt->cddb_arg,"query[") == opt->cddb_arg))
+            if (strncmp(opt->cddb_arg, "query", 5)==0)
             {
-              int ambigous = parse_query_arg(opt,opt->cddb_arg);
-              if (ambigous)
+              //only do freedb search for the first time
+              if (j == 0)
               {
-                print_warning("freedb server format ambigous !");
+                int ambigous = parse_query_arg(opt,opt->cddb_arg);
+                if (ambigous)
+                {
+                  print_warning("freedb query format ambigous !");
+                }
+                do_freedb_search(data);
               }
-              do_freedb_search(data);
+
+              //we get the splitpoints from the file
+              mp3splt_put_cddb_splitpoints_from_file(state, MP3SPLT_CDDBFILE, &err);
+              process_confirmation_error(err, data);
             }
             else
               //here we have cddb file
@@ -1914,7 +2101,7 @@ int main(int argc, char *argv[])
             for (i = 0;i < data->number_of_splitpoints; i++)
             {
               long point = data->splitpoints[i];
-              err = mp3splt_append_splitpoint(state, point, NULL);
+              err = mp3splt_append_splitpoint(state, point, NULL, SPLT_SPLITPOINT);
               process_confirmation_error(err, data);
             }
           }
